/**
 * synopsis: Generates an JWM menu xml segment based on an XDG menu structure
 * purpose: The purpose of this utility is to keep the JWM menu
 *          synchronized with the XDG configuration
 * usage: jwm-xdgmenu <xdgMenuFile>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 *
 * Dependencies:
 *   gnome-menus 2.12
 *   glib
 */

/**
 * Enhancements: Double spaces before menu entry, adjustable menu height & 
 * icon extension checks by Patriot, May 2009 
 */
 
#include <glib.h>
#include <glib/gprintf.h>
#include <gmenu-tree.h>


/*=============================================================================
 * Declarations
 */
static void show_help();
static void process_directory(GMenuTreeDirectory *directory);
static void process_entry(GMenuTreeEntry *entry);
static void process_separator(GMenuTreeSeparator *entry);
static int menuheight = 16;

/*=============================================================================
 * Main Function
 */
int
main (int argc, char **argv)
{
  /** Patriot: added menu height option
   *  Gross assumption: User know what they're doing
   */
  if (strcmp (argv[1], "-m") == 0)
  {
  	menuheight = atoi(argv[2]);
  }
  else
    if ((argc != 2)
      || (strcmp (argv[1], "-h") == 0)
      || (strcmp (argv[1], "-help") == 0)
      || (strcmp (argv[1], "--help") == 0))
    {
      show_help();
      return (1);
    }

  //GMenuTree *menuTree = gmenu_tree_lookup (argv[1],  GMENU_TREE_FLAGS_NONE );
  GMenuTree *menuTree = gmenu_tree_lookup (argv[--argc],  GMENU_TREE_FLAGS_NONE );

  GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

  process_directory(rootDirectory);

  gmenu_tree_item_unref (rootDirectory);

  return 0;

}

/*=============================================================================
 * Shows usage parameters
 */
void
show_help()
{
  printf ("Creates a JWM menu entries from a given XDG menu structure.\n");
  printf ("Usage:\n");
  printf ("  jwm-xdgmenu [-m menuheight] xdgmenufile\n");
  printf ("\n");
  printf ("  xdgmenufile: Fully qualified path to the XDG menu file. COMPULSORY parameter.\n");
  printf ("  -m menuheight: optional menu height. Must be spaced.");
  printf ("\n");
  printf ("For example:\n");
  printf ("  jwm-xdgmenu -m 20 /etc/xdg/menus/applications.menu\n");
}

/*=============================================================================
 * This function processes a directory entry and all it's child nodes
 */
void
process_directory(GMenuTreeDirectory *directory)
{
	int hasSeparator = 0;

  /** Patriot: icon extension check. Default to .png **/
  gchar *gtd_icon = g_strdup(gmenu_tree_directory_get_icon(directory));
  gchar *gtd_icon_ext = gtd_icon;
  if ( ! (g_str_has_suffix(gtd_icon, ".png") ||
		  g_str_has_suffix(gtd_icon, ".xpm") ||
		  g_str_has_suffix(gtd_icon, ".jpg")) )
    {
      gtd_icon_ext = g_strconcat(gtd_icon, ".png", NULL);
      g_free(gtd_icon);
      gtd_icon = gtd_icon_ext;
    }

  /*BK height param added... */
  /** Patriot: adjustable menu height */
  g_printf("  <Menu label=\"  %s\" icon=\"%s\" height=\"%d\">\n",
            gmenu_tree_directory_get_name(directory),
            gtd_icon,
            menuheight);

  g_free(gtd_icon);

  GMenuTreeItemType entryType;

  GSList *entryList = gmenu_tree_directory_get_contents (directory);

  GSList *l;

  for (l = entryList; l; l = l->next)
  {

    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

	switch (entryType)
	{
		case GMENU_TREE_ITEM_DIRECTORY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_directory(GMENU_TREE_DIRECTORY(item));
			break;
		case GMENU_TREE_ITEM_ENTRY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_entry(GMENU_TREE_ENTRY(item));
			break;
		case GMENU_TREE_ITEM_SEPARATOR:
			hasSeparator = 1;
			break;
	}

    gmenu_tree_item_unref (item);

  }
  g_printf("  </Menu>\n");
  g_slist_free (entryList);
}

/*=============================================================================
 * This function adds an application entry
 */
void
process_entry(GMenuTreeEntry *entry)
{

  /** Patriot: icon extension check. Default to .png **/
  gchar *gte_icon = g_strdup(gmenu_tree_entry_get_icon(entry));
  gchar *gte_icon_ext = gte_icon;
  if ( ! (g_str_has_suffix(gte_icon, ".png") ||
		  g_str_has_suffix(gte_icon, ".xpm") ||
		  g_str_has_suffix(gte_icon, ".jpg")) )
    {
      gte_icon_ext = g_strconcat(gte_icon, ".png", NULL);
      g_free(gte_icon);
      gte_icon = gte_icon_ext;
    }

  g_printf("    <Program label=\"  %s\" icon=\"%s\">%s</Program>\n",
            gmenu_tree_entry_get_name(entry),
            gte_icon,
            gmenu_tree_entry_get_exec (entry));

  g_free(gte_icon);

}

/*=============================================================================
 * This function adds a separator
 */
void
process_separator(GMenuTreeSeparator *entry)
{
  g_printf("  <Separator/> \n");
}

/*=============================================================================
 */
